/* DOCUMENTATION
* MODULE: vTable.h
*
* Copyright 2000, 2002 Control Technology Corporation as an unpublished work.
* All Rights Reserved.
*
* MODULE DESCRIPTION
* Interface module for vTable.c.
*/

#ifndef uTable_H
#define uTable_H
#ifdef   __cplusplus
extern   "C" {
#endif
//#include "tx_api.h"
//#include "QSMain.h"
//#include "log.h"

// Network Function parameters

/****************************************************************
*  struct USERCONNECTION 									    *
*																*
*  This parameter block must be filled out prior to making any  *
*   Network calls.  This block should be maintained for the     *
*   life of the connection and treated as a connection HANDLE   *
****************************************************************/
typedef struct userconnection
{
	int type;		// type of connection... NETWORK_TYPE_UDP...
	int state;		// State of connection
	unsigned char *packetBuf;	// pointer to packet (RX/TX)
	int length;		// number of bytes in Packet buffer (RX/TX)
	int srcPort;		// our port to open socket with, 0 for any
	unsigned long destIp;	// address to connect to or RX is from
	int destPort;		// port to connect to or RX is from
	void (*StateCallback)(void *ptr);  // callback for State change
	void *lhSocket;	// Socket information
	void *lhThread;	// RX Thread information
} USERCONNECTION;

// USERCONNECTION type
#define NETWORK_TYPE_UDP 1
#define NETWORK_TYPE_TCP_CLIENT 2	// not implemented
#define NETWORK_TYPE_TCP_SERVER 3	// not implemented

// USERCONNECTION state phases
#define USER_DISCONNECTED 0	// socket closed, disconnected
#define USER_CONNECTING 1	// thread spawned awaiting connection, StateCallback will occur
#define USER_CONNECTED 2	// connection successful, thread in rx mode
#define USER_DATA_AVAILABLE 3	// rx data available
#define USER_DATAQUEUED 4	// TX data transferred to driver
#define USER_FAILED 5		// operation requested failed


/* Copy to end of file, below to the uTable.h in User Functions *
*																*
*****************************************************************
*  QUICKSTEP OS VIRTUAL FUNCTION TABLE PROTOTYPE DEFINITIONS	*
*																*
*  Virtual function table for access to User functions.			*
*  This table is initialized by user within their main() 		*
*  function, upon initial loading into memory and prior to		*
*  operation													*
****************************************************************/

/********* Register Access ***************/
typedef RETVAL (*REGISTER_GET)( TASK *task, UINT16 RegNum, INT32 *RegVal);
typedef RETVAL (*REGISTER_PUT)( TASK *task, UINT16 RegNum, INT32 RegVal);

/********* Communications ****************/
typedef RETVAL (*COMM_SENDMSG)( INDX port, UINT8 *msg, UINT16 size);
typedef RETVAL (*COMM_GENERICCMD)( INDX port, UINT8 *msg, UINT16 *size);
typedef RETVAL (*COMM_GET_RQST)( INDX port, UINT8 **buf, UINT16 *size);
typedef int (*COMM_NETWORKOPEN)(USERCONNECTION *ptr);	
typedef void (*COMM_NETWORKCLOSE)(USERCONNECTION *ptr);	
typedef int (*COMM_NETWORKSEND)(USERCONNECTION *ptr);	

/************ Motion Routines ****************/
typedef	UINT16 (*MOTION_GET_NUMMOTORS)(void);
typedef	RETVAL (*MOTION_GET_ATTRIBUTE)( UINT8 MotorNum, UINT8 Attribute, STDVAL *value);
typedef	RETVAL (*MOTION_PUT_ATTRIBUTE)( UINT8 MotorNum, UINT8 Attribute, STDVAL value);
typedef	RETVAL (*MOTION_READY)( UINT8 MotorNum, STDVAL *value);
typedef RETVAL (*MOTION_SIMPLE_COMMAND)( UINT8 MotorNum, UINT8 Command);

/*********** System Functions ***************/
typedef unsigned long (*GET_SYSTEM_TICS)(void);		
/*********** Base Threading Functions (Advanced users only) ************/
typedef unsigned int (*THREAD_CREATE)( TX_THREAD *thread_ptr,       /* Task Structure */
						/* Task Name */
                        char *name_ptr,
						/* Routine and Parameter to pass */
                        void (*entry_function)(unsigned long), unsigned long entry_input,
                        /* Stack start and length */
                        void *stack_start, unsigned long stack_size,
                        /* Priority and Threshold */
                        unsigned int priority, unsigned int preempt_threshold, 
						/* time slice */
                        unsigned long time_slice, unsigned int auto_start);

typedef unsigned int (*THREAD_SUSPEND)(TX_THREAD *thread_ptr);  
typedef unsigned int (*THREAD_RESUME)(TX_THREAD *thread_ptr);   
typedef unsigned int (*THREAD_SLEEP)(unsigned long tics);	
typedef unsigned int (*MUTEX_CREATE)(TX_MUTEX *mutex_ptr, char *name,unsigned int inherit);  
typedef unsigned int (*MUTEX_GET)(TX_MUTEX *mutex_ptr, unsigned long wait_option);  
typedef unsigned int (*MUTEX_PUT)(TX_MUTEX *mutex_ptr);	
typedef unsigned int (*MUTEX_DELETE)(TX_MUTEX *mutex_ptr);	

/*********** Diagnostics ***************/
typedef RETVAL (*LOGEVENT)(EventCode event, long parameter);
typedef RETVAL (*LOGEVENTPLUS)(EventCode event, long firstParameter, long secondParameter);
typedef int (*PRINTF)(const char *format, ...);
typedef int (*SPRINTF)(char *buffer, const char *format, ...);

/****** CTCONLY:  System Memory allocation to support malloc() and free()  *******/
typedef unsigned int (*BYTE_ALLOCATE)(TX_BYTE_POOL *pool_ptr,
			void **memory_ptr,unsigned long memory_size,
			unsigned long wait_option); 
typedef unsigned int (*BYTE_RELEASE)(void *memory_ptr); 
typedef unsigned int (*BYTE_POOL_CREATE)(TX_BYTE_POOL *pool_ptr,
			char *name_ptr, void *pool_start, 
			unsigned long pool_size);
typedef unsigned int (*BYTE_POOL_DELETE)(TX_BYTE_POOL *pool_ptr);

/****** CTCONLY:  Interrupt Control  *******/
typedef unsigned int (*INTERRUPT_CONTROL)(unsigned int);

/****** Step Atomicity  *******/
typedef RETVAL (*ENTER_STEP_ATOMICITY)();
typedef RETVAL (*EXIT_STEP_ATOMICITY)();

/****** Watchdog timer control  *******/
typedef void (*FIREWATCHDOG)();

// Resource Filter Function parameters

// Type of resources to filter
#define RESOURCE_NONE 0
#define RESOURCE_ANALOGIN 1
#define RESOURCE_ANALOGOUT 2
#define RESOURCE_MOTION 3
#define RESOURCE_DIGITALIN 4		// NOT SUPPORTED
#define RESOURCE_DIGITALOUT 5		// NOT SUPPORTED
#define RESOURCE_DATATABLE 6
#define RESOURCE_REGISTER 7

// Filter mode and enabled modes
#define RESOURCE_READ 1		// read only
#define RESOURCE_WRITE 2	// write only

/****************************************************************
*  struct resourceInfo - addResourceFilter parameter block      *
*																*
*  This parameter block must be filled out prior to registering *
*  a callback function for a Quickstep resource.  A pointer to  *
*  it is passed to the addResourceFilter() function             *
****************************************************************/
typedef struct resourceInfo {
	int type;		// Type of resource to add filter to, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int mode;		// Type of access to invoke filter on when accessed, RESOURCE_READ, RESOURCE_WRITE
	int start;		// Start number of resource (setting a range)
	int end;		// End number of resource, make same as start if only one (setting a range).
} RESOURCE_INFO;

/****************************************************************
*  struct filterparams - Resource Filter Function callback block*
*																*
*  Registered resource functions are called with a pointer      *
*  to this structure in order to pass information important     *
*  to the callback funciton.  This is typically information     *
*  with regards to the type of access being performed on the    *
*  resource, resource number, type, etc                         *
****************************************************************/
typedef struct filterparams
{
	int type;			// Type of resource calling, RESOURCE_ANALOGIN, RESOURCE_ANALOGOUT, etc...
	int resourceNum;    // Quickstep resource number for this resource, 1 to 65536 for registers, 0 to nn analog, ...
	int mode;			// Type of access being performed, RESOURCE_READ, RESOURCE_WRITE
	int row;			// If RESOURCE_DATATABLE then the row and column values being accessed are passed
	int col;
	void *option;		// Reserved for future expansion
}FILTERPARAMS;

/****************************************************************
* FILTER_FUNCTION                               			    *
*																*
* Type definition for Resource Callback function			    *
*																*
* PARAMETERS:													*
* void *handle - handle returned by addUserResourceFilter when  *
*                function was registered                        *
* FILTERPARAMS *params - Access information block to detail     *
*                        what is being done                     *
* STDVAL value - Current value being read or written.           *
* RETVAL *status - pointer to status code that will be returned *
*                  to Quickstep.  Leaving it unchanged will     *
*                  default to SUCCESS.  Use only for defined    *
*                  errors.                                      *
*																*
* RETURNS:														*
* int - new value to return to Quickstep on a read or value to  *
*       write, if no change then return passed "value"          *
*****************************************************************/
typedef int (*FILTER_FUNCTION)(void *handle, FILTERPARAMS *params, STDVAL value, RETVAL *status);

// Virtual Function table prototype for adding and removing a filter/trigger function
typedef void *(*ADD_RESOURCE_FILTER)(RESOURCE_INFO *rsc, FILTER_FUNCTION func);	
typedef RETVAL (*REMOVE_RESOURCE_FILTER)(void *handle);

/****************************************************************
*  struct vtableQS - Quickstep OS Function table				*
*																*
*  Virtual function table for access to Quickstep functions.	*
*  This table is initialized by Quickstep OS at powerup.		*
****************************************************************/
typedef struct vtableQS
{
	REGISTER_GET            regRead;				// read a Quickstep register
	REGISTER_PUT            regWrite;				// write a Quickstep register

	COMM_SENDMSG            commSendMsg;			// Send a string out a serial port
	COMM_GENERICCMD         commGenericCmd;			// Send a command to a serial port
	COMM_GET_RQST           commReadMsg;			// Read a message from a serial port

	ADD_RESOURCE_FILTER     addResourceFilter;		// add a resource filter to the access list
	REMOVE_RESOURCE_FILTER  removeResourceFilter;	// remove a resource filter from the access list

	GET_SYSTEM_TICS         systemTics;				// Returns number of tics since powerup, in ms.

	PRINTF                  printf;					// printf function redirected to UDP debugger output screen
	SPRINTF                 sprintf;				// sprintf function used by Quickstep, protected by mutex
	LOGEVENTPLUS            logEventPlus;			// Log a predetermined value type and value/string to the event log
	LOGEVENT                logEvent;				// Log a low level value into the debug event log
	FIREWATCHDOG			fireWatchdog;			// Reset a watchdog timer, Quickstep calls it continually.
													// If for any reason not return control must be called in
													// less than 86 ms. or reset will occur.

	ENTER_STEP_ATOMICITY    enterStepAtomicity;		// Obtain ownership of step atomicity
	EXIT_STEP_ATOMICITY     exitStepAtomicity;		// Release ownership of step atomicity...

	/***************************************************************/
	// below virtual functions are for advanced use only and not 
	// offered for general support
	/***************************************************************/
	MOTION_GET_NUMMOTORS    Motion_GetNumMotors;	// Get number of motion objects in system
	MOTION_GET_ATTRIBUTE    Motion_Get_Attribute;
	MOTION_PUT_ATTRIBUTE    Motion_Put_Attribute;
	MOTION_READY            Motion_Ready;			// Get state of motion object
	MOTION_SIMPLE_COMMAND   Motion_Simple_Command;

	THREAD_CREATE           thread_create;			// Create a new thread
	THREAD_SUSPEND          thread_suspend;			// Suspend an existing thread
	THREAD_RESUME           thread_resume;			// Allow a suspended thread to resume operation
	THREAD_SLEEP            thread_sleep;			// Sleep for the specified number of tics (1ms/tic)
	MUTEX_CREATE            mutex_create;			// Create a mutex object
	MUTEX_GET               mutex_get;				// get control of the mutex
	MUTEX_PUT               mutex_put;				// give control of the mutex back
	MUTEX_DELETE            mutex_delete;			// Destroy the mutex, giving all memory back

	BYTE_ALLOCATE           tx_byte_allocate;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_RELEASE            tx_byte_release;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_POOL_CREATE        tx_byte_pool_create;	// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	BYTE_POOL_DELETE        tx_byte_pool_delete; 	// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES
	INTERRUPT_CONTROL    	interruptControl;		// NOT TO BE USED EXCEPT FOR INTERNAL ROUTINES

	COMM_NETWORKOPEN		commNetworkOpen;		// Open a network connection/socket spawning RX thread	
	COMM_NETWORKCLOSE		commNetworkClose;		// Close a network connection
	COMM_NETWORKSEND		commNetworkSend;		// Send data on a network connection

} VTABLE_QS;

/****************************************************************
*  USER VIRTUAL FUNCTION TABLE PROTOTYPE DEFINITIONS			*
*																*
*  Virtual function table hooks for access by Quickstep OS 		*
*  to call if defined during strategic points of system			*
*  execution.													*
*  This table is initialized by user within their main() 		*
*  function, with their 'C' functions, as needed, else place	*
*  a NULL in the table entry									*		
****************************************************************/
typedef int   (*USER_INITIALIZATION)(void);       	// Called when started, = main
typedef int   (*USER_OPERATION)(void *);          	// Called to invoke a user operation

typedef void  (*TIMER_INTERRUPT_HOOK)();			// invoked every timer tic, don't stay here long!!!
typedef int   (*TASK_LOOP_HOOK)(SYSMODE state);	  	// invoked after all Quickstep tasks have executed, round robin
typedef	int   (*SUPERTASK_HOOK)(SYSMODE state);	  	// invoked after each Quickstep task
typedef	int   (*SERVICELOOP_HOOK)(SYSMODE state);	// invoked after each Quickstep task
typedef void  (*END_ALL_TASKS_HOOK)();		        // invoked if all tasks are told to end.
typedef void  (*USER_SHUTDOWN)();		        	// invoked if about to load another user module on top of this one.


/****************************************************************
*  struct vtableUser - User Defined Function table				*
*																*
*  Virtual function table for access to User functions.			*
*  This table is initialized by user within their main() 		*
*  function, upon initial loading into memory and prior to		*
*  operation													*
****************************************************************/
#define VALID_USER_SIGNATURE  0x10adf7L		// Special identifier at start of User table, required
#define USER_VERSION_MASK     0xffff0000L
#define USER_ENTRIES_MASK     0x000000ffL

#define USER_VERSION_REQUIRED 0x00010000L	// Not currently used
#define USER_ENTRIES_REQUIRED 8				// Number of function entries in the vtableUser structure
typedef struct vtableUser
{
  	long                  signature;	   	// Unique value to verify table is correct type
  	long                  version;			// Version control information to ensure compatibility
  	USER_INITIALIZATION   initialize;		// User routine to be called at start, main() function
  	USER_OPERATION        function;			// NOT USED
	TIMER_INTERRUPT_HOOK  timerTic;			// Invoked on every timer tic (1ms) if defined
	TASK_LOOP_HOOK        taskLoop;			// Invoked as a normal Quickstep step if defined
	SUPERTASK_HOOK        superTaskLoop;	// Invoked after each Quickstep step, like a super task
	SERVICELOOP_HOOK      serviceLoopHook;	// Invoked outside the Quickstep main loop when steps not running
	END_ALL_TASKS_HOOK    endAllTasks;		// Invoked if all tasks are being shutdown by a Quickstep Cancel command
	USER_SHUTDOWN			userShutdown;	// Invoked when a new "C" module is being loaded for cleanup purposes
} VTABLE_USER;

	
// Prototype declaration of function used to retrieve the core Quickstep OS table.
// Quickstep OS invokes this via a software interrupt and returns the value to the
// User 'C' function.
extern const VTABLE_QS *getCoreTable(void);

#ifdef   __cplusplus
}
#endif

#endif



